local VERSION = tonumber(("$Revision: 43089 $"):match("%d+"))

if uab.revision < VERSION then
	uab.version = "r" .. VERSION
	uab.revision = VERSION
	uab.date = ("$Date: 2007-07-04 17:50:18 -0400 (Wed, 04 Jul 2007) $"):match("%d%d%d%d%-%d%d%-%d%d")
end

local uabActivate = uab:NewModule("Activate", "AceConsole-2.0")

uabActivate.core = uab

--[[
Constant menu/slash commands.
--]]

uabActivate.options = {
    type = 'group',
    args = {},
}

function uabActivate:OnInitialize()
    self:RegisterEvent("UnitActionBars_RefreshActivators")

    self:RefreshActivatorOptions()

    self:RegisterChatCommand({ "/unitactivate", "/uaba" }, self.options)
end

function uabActivate:UnitActionBars_RefreshActivators()
    self:RefreshActivatorOptions()
end

function uabActivate:RefreshActivatorOptions()
    self.options.args = {}
    
    self.options.args.closeall = {
        type = 'execute',
        name = "Close all",
        desc = "Close any action buttons for all activators",
        disabled = function() return InCombatLockdown() end,
        func = function()
            self:CloseAllActivators()
        end,
		order = 5,
    }

    self.options.args.open = {
        type = 'group',
        name = "Open",
        desc = "Open action buttons for an activator on a unit frame",
        disabled = function() return InCombatLockdown() end,
		order = 10,
        args = {},
    }

    -- For each frame group, add a set of activator options.

    self.options.args.open.args["default"] = self:CreateFrameGroupOptions("default")
    self.options.args.open.args["default"].order = 10

    local order = 20
    for key, val in pairs(uabFrameGroup.modules) do
        self.options.args.open.args[key] = self:CreateFrameGroupOptions(key)
        self.options.args.open.args[key].order = order
        order = order + 10
	end
end

function uabActivate:CreateFrameGroupOptions(fgName)
    local options
    
    options = {
        type = 'group',
        name = fgName,
        desc = "Activate actions on the frame in the "..fgName.." frame group",
        args = {
        },
    }

    local order = 10	
	for key, val in pairs(self.core.db.profile.activators) do
        options.args[key] = self:CreateActivatorOptions(fgName, key)
        options.args[key].order = order
        order = order + 10
	end
    
    return options
end

function uabActivate:CreateActivatorOptions(fgName, name)
    local options
    
    options = {
        type = 'group',
        name = name,
        desc = "Activator action unit groups",
        args = {
            player = {
                type = 'execute',
                name = "Player",
                desc = "Player frame",
                disabled = function() return InCombatLockdown() end,
                order = 5,
                func = function()
                    self:OpenActivatorActions(fgName, name, "player")
                end,
            },
            party = {
                type = 'text',
                name = "Party",
                usage = "1-4",
                desc = "Party frame #",
                disabled = function() return InCombatLockdown() end,
                order = 10,
                get = function() return end,
                set = function(pnum)
                    self:OpenActivatorActions(fgName, name, "party"..pnum)
                end,
                validate = function(pnum)
                    local n = tonumber(pnum)
                    return n >= 1 and n <= 4
                end,
            },
            partypet = {
                type = 'text',
                name = "Party pet",
                usage = "1-4",
                desc = "Party pet frame #",
                disabled = function() return InCombatLockdown() end,
                order = 15,
                get = function() return end,
                set = function(pnum)
                    self:OpenActivatorActions(fgName, name, "partypet"..pnum)
                end,
                validate = function(pnum)
                    local n = tonumber(pnum)
                    return n >= 1 and n <= 4
                end,
            },
            raid = {
                type = 'text',
                name = "Raid",
                usage = "1-4",
                desc = "Raid unit #",
                disabled = function() return InCombatLockdown() end,
                order = 20,
                get = function() return end,
                set = function(rnum)
                    self:OpenActivatorActions(fgName, name, "raid"..rnum)
                end,
                validate = function(rnum)
                    local n = tonumber(rnum)
                    return n >= 1 and n <= 40
                end,
            },
            target = {
                type = 'execute',
                name = "Target",
                desc = "Target frame",
                disabled = function() return InCombatLockdown() end,
                order = 25,
                func = function()
                    self:OpenActivatorActions(fgName, name, "target")
                end,
            },
            tot = {
                type = 'execute',
                name = "ToT",
                desc = "ToT frame",
                disabled = function() return InCombatLockdown() end,
                order = 30,
                func = function()
                    self:OpenActivatorActions(fgName, name, "targettarget")
                end,
            },
            tott = {
                type = 'execute',
                name = "ToT Target",
                desc = "ToT target frame",
                disabled = function() return InCombatLockdown() end,
                order = 35,
                func = function()
                    self:OpenActivatorActions(fgName, name, "targettarget")
                end,
            },
            focus = {
                type = 'execute',
                name = "Focus",
                desc = "Focus frame",
                disabled = function() return InCombatLockdown() end,
                order = 40,
                func = function()
                    self:OpenActivatorActions(fgName, name, "focus")
                end,
            },
            focustarget = {
                type = 'execute',
                name = "Focus target",
                desc = "Focus target frame",
                disabled = function() return InCombatLockdown() end,
                order = 45,
                func = function()
                    self:OpenActivatorActions(fgName, name, "focustarget")
                end,
            },
        },
    }

    return options
end

function uabActivate:OpenActivatorActions(fgName, name, unit)
    local args = {
        framegroup = fgName,
        activator = uab.main:GetActivator(name),
        unit = unit,
    }
    
    uab.print("Opening activator actions for the "..name.." activator on frames for the "..unit.." unit")
    uab.classes.UnitGroupControl:CallForAllControls(self.OpenIfForActivatorAndUnitOnFrameGroupFrame, self, args)
    if args.ctrl then
        
        -- This is a UnitGroupControl for the desired activator and unit.
        -- Open the controls by simulating a click on that control.
        -- Each control has a state associated with it.  That state drives
        -- what happens when that button is clicked.  It controls how the
        -- parent is set, how the secure header is re-parented, and how the
        -- action buttons are positioned, scaled, and so on.  We indicate on
        -- the click that the button associated with the activator is the
        -- button causing the click.  The only differences between this call
        -- to Click and the one that happens when a real click happens are:
        --
        -- 1) The modifiers (shift, ctrl, etc.) are not pressed as they would be
        -- on a normal click.  However, the UnitGroupControl does not use that
        -- information.  It is just used when the *real* click happens on the
        -- associated unit frame, so that the secure state engine knows how to
        -- direct the click to take the correct action (a click on the control).
        --
        -- 2) The mouse is not positioned (necessarily) where it would be when
        -- the frame is clicked.  So, if the layout indicates cursor-positioning
        -- then the action buttons will show up relative to the cursor, not the
        -- frame they are associated with.
        
        args.ctrl.frame:Click(args.activator.data.button)
    end
end

function uabActivate.OpenIfForActivatorAndUnitOnFrameGroupFrame(ctrl, self, args)
    if ctrl:ActiveOnFrame() and ctrl:GetActivator() == args.activator and ctrl.unitframeunit == args.unit then
        local frame = ctrl.unitframe
        local framegroup = uabFrameGroup:GetFrameGroup(frame, false)

        if framegroup.name == args.framegroup then
            -- This is a UnitGroupControl for the desired activator and unit.
            
            if not args.ctrl then
                args.ctrl = ctrl
            else
                -- Have to determine whether this is the control we want to
                -- pick to open action buttons for.
                
                -- if the unit frame this control is associated with is visible
                -- then consider replacing the one we already picked.  But only
                -- if that one is not visible.
                
                if ctrl.unitframe:IsVisible() and not args.ctrl.unitframe:IsVisible() then
                    args.ctrl = ctrl
                end
            end
        end
    end
end

function uabActivate:CloseAllActivators()
    uab.classes.SecureHeader:HideAllHeaders()
end

